package com.stormtales.notevault.data.repositories;

import android.app.Application;
import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import androidx.lifecycle.LiveData;
import com.stormtales.notevault.data.MusicDatabase;
import com.stormtales.notevault.data.dao.SongDao;
import com.stormtales.notevault.data.entities.NoteSheet;
import com.stormtales.notevault.data.entities.Song;
import com.stormtales.notevault.data.sync.SyncStatus;
import com.stormtales.notevault.utils.NoteSheetsUtil;

import java.io.IOException;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class SongRepository {
    private SongDao songDao;
    public SongRepository(Context context) {
        MusicDatabase database = MusicDatabase.getDatabase(context);
        songDao = database.getSongTable();
    }
    public void insert(Song song, Callback<Long> callback) {
        Executors.newSingleThreadExecutor().execute(() -> {
            long result = songDao.insert(song);
            Handler mainHandler = new Handler(Looper.getMainLooper());
            mainHandler.post(()-> callback.onResult(result));
        });
    }

    public interface LoadHomeViewModelCallback<T> {
        void onResult(T result);
    }

    public void getAllSongs(LoadHomeViewModelCallback<List<Song>> callback) {
        ExecutorService executorService = Executors.newSingleThreadExecutor();
        executorService.execute(() -> {
            List<Song> songs = songDao.getAllSongs();
            Handler mainHandler = new Handler(Looper.getMainLooper());
            mainHandler.post(()-> callback.onResult(songs));
        });
    }

    public void updateSong(Song song) {
        Executors.newSingleThreadExecutor().execute(() -> {
            songDao.update(song);
        });
    }

    public void insertNoteSheets(List<NoteSheet> noteSheets) {
        Executors.newSingleThreadExecutor().execute(() -> {
            songDao.insert(noteSheets);
        });
    }

    public interface Callback<T> {
        void onResult(T result);
    }

    public void deleteSong(Song song) {
        Executors.newSingleThreadExecutor().execute(() -> {
            song.setSyncStatus(SyncStatus.DELETED);
            songDao.update(song);

            List<NoteSheet> noteSheets = songDao.getNoteSheetsBySong(song.getLocalID());
            for(NoteSheet noteSheet : noteSheets) {
                try {
                    NoteSheetsUtil.deleteNoteSheet(noteSheet.getLocalFileName());
                } catch (IOException e) {
                    throw new RuntimeException(e);
                }
            }
            songDao.deleteNoteSheets(noteSheets);
        });
    }

    public void getNoteSheetFilesBySongID(int songID, Callback<List<String>> callback) {
        Executors.newSingleThreadExecutor().execute(()-> {
            List<String> noteSheetFiles = songDao.getNoteSheetFilesBySongID(songID);
            callback.onResult(noteSheetFiles);
        });
    }
}
