package com.stormtales.notevault.data.repositories;

import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import com.stormtales.notevault.data.MusicDatabase;
import com.stormtales.notevault.data.dao.SongDao;
import com.stormtales.notevault.data.entities.Song;
import com.stormtales.notevault.data.sync.SyncStatus;
import com.stormtales.notevault.network.sync.models.BatchCreateResponse;
import com.stormtales.notevault.network.sync.models.CreateResponse;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;

public class SongSyncRepository {
    private SongDao songDao;
    public SongSyncRepository(Context context) {
        MusicDatabase database = MusicDatabase.getDatabase(context);
        songDao = database.getSongTable();
    }

    public void loadCreatedSongs(LoadDataCallback<List<Song>> callback) {
        Executors.newSingleThreadExecutor().execute(() -> {
            List<Song> createdSongs = songDao.getSongsBySyncStatus(SyncStatus.CREATED);
            Handler mainHandler = new Handler(Looper.getMainLooper());
            mainHandler.post(()-> callback.onResult(createdSongs));
        });
    }

    public void markCreatedSongsAsSynced(BatchCreateResponse createdSongs) {
        Executors.newSingleThreadExecutor().execute(() -> {
            List<Integer> localIDs = createdSongs.getCreateResponses().stream().map(CreateResponse::getLocalID).collect(Collectors.toList());
            List<Song> requestedSongs = songDao.getSongsByLocalIDs(localIDs);

            for(Song song : requestedSongs) {
                song.setSyncTime(LocalDateTime.now());
                song.setSyncStatus(SyncStatus.SYNCED);

                for(CreateResponse createResponse : createdSongs.getCreateResponses()) {
                    if(createResponse.getLocalID() == song.getLocalID()) {
                        song.setServerID(createResponse.getServerID());
                        break;
                    }
                }
            }
            songDao.updateSongs(requestedSongs);
        });
    }

    public void markSongsAsSynced(List<Song> songs) {
        for(Song song : songs) {
            song.setSyncStatus(SyncStatus.SYNCED);
        }
    }

    public interface LoadDataCallback<T> {
        void onResult(T result);
    }
}
