package com.stormtales.notevault.network.auth;

import android.content.Context;
import android.text.TextUtils;
import com.stormtales.notevault.network.APICallback;
import com.stormtales.notevault.network.NetworkModule;
import com.stormtales.notevault.network.StatusResponse;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class AuthService {

    private final AuthAPI authAPI;
    private final TokenManager tokenManager;

    public AuthService(Context context) {
        this.authAPI = NetworkModule.getRetrofitInstance(context).create(AuthAPI.class);
        this.tokenManager = new TokenManager(context);
    }

    public void performLogin(String email, String password, LoginCallback callback) {
        LoginRequest loginRequest = new LoginRequest(email, password);

        authAPI.login(loginRequest).enqueue(new Callback<LoginResponse>() {
            @Override
            public void onResponse(Call<LoginResponse> call, Response<LoginResponse> response) {
                if (response.isSuccessful() && response.body() != null) {
                    String token = response.body().getToken();
                    saveToken(token);
                    // Erfolgsrückmeldung an den Callback senden
                    callback.onSuccess(response.body());
                } else {
                    // Fehlermeldung an den Callback senden
                    callback.onError("Login fehlgeschlagen. Überprüfe Benutzername und Passwort.");
                }
            }

            @Override
            public void onFailure(Call<LoginResponse> call, Throwable t) {
                // Netzwerkfehler an den Callback senden
                callback.onError("Netzwerkfehler: " + t.getMessage());
            }
        });
    }

    public void performRegistration(String email, String username, String password, APICallback callback) {
        RegisterRequest registerRequest = new RegisterRequest(username, password, email);
        authAPI.registration(registerRequest).enqueue(new Callback<StatusResponse>() {

            @Override
            public void onResponse(Call<StatusResponse> call, Response<StatusResponse> response) {
                if(response.isSuccessful() && response.body() != null) {
                    callback.onSuccess();
                } else {
                    callback.onError("Registration fehlgeschlagen. Überprüfe Benutzername und Passwort.");
                }
            }

            @Override
            public void onFailure(Call<StatusResponse> call, Throwable throwable) {
                callback.onError("Netzwerkfehler: " + throwable.getMessage());
            }
        });
    }

    private void saveToken(String token) {
        tokenManager.saveToken(token);
    }

    public String getToken() {
        return tokenManager.getToken();
    }

    public void logout() {
        tokenManager.clearToken();
    }

    public boolean isLoggedIn() {
        return !TextUtils.isEmpty(tokenManager.getToken());
    }

    public interface LoginCallback {
        void onSuccess(LoginResponse loginResponse);
        void onError(String error);
    }
}
