package com.stormtales.notevault.network.sync;

import android.content.Context;
import com.stormtales.notevault.data.entities.Song;
import com.stormtales.notevault.network.NetworkModule;
import com.stormtales.notevault.network.auth.AuthAPI;
import com.stormtales.notevault.network.sync.models.BatchCreateResponse;
import com.stormtales.notevault.network.sync.models.SongCreateBatchRequest;
import com.stormtales.notevault.network.sync.models.SongCreateRequest;
import com.stormtales.notevault.ui.gallery.GalleryViewModel;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

import java.util.ArrayList;
import java.util.List;

public class SongSyncService {
    private final SongSyncAPI songSyncAPI;
    private GalleryViewModel syncStatusViewModel;

    public SongSyncService(Context context) {
        songSyncAPI = NetworkModule.getRetrofitInstance(context).create(SongSyncAPI.class);
    }

    public void syncCreatedSongs(List<Song> songs, SongSyncModule.FinishSongSyncingCallback callback) {
        List<SongCreateRequest> createRequests = new ArrayList<>();
        for(Song song : songs) {
            createRequests.add(new SongCreateRequest(song));
        }
        SongCreateBatchRequest songCreateBatchRequest = new SongCreateBatchRequest(createRequests);
        songSyncAPI.syncCreatedSongs(songCreateBatchRequest).enqueue(new Callback<BatchCreateResponse>() {
            @Override
            public void onResponse(Call<BatchCreateResponse> call, Response<BatchCreateResponse> response) {
                callback.finishSongSyncing(response.body());
            }

            @Override
            public void onFailure(Call<BatchCreateResponse> call, Throwable throwable) {

            }
        });
    }
}
